@@ -42,11 +42,42 @@ class MissingRequiredArgument(CmdBaseException):
         super().__init__(self.message)
 
 
+class Parser:
+    def __init__(self, text, prefix):
+        self.text = text
+        self.index = -1
+        self.prefix = prefix
+        self.char = None
+        self.args = None
+
+        if self.text:
+            self._parse()
+
+    def _shift(self):
+        self.index += 1
+        self.char = self.text[self.index] if self.index < len(self.text) else None
+
+    def _parse(self):
+        self._shift()
+        space_count = 0
+        while self.char is not None:
+            if self.char == " ":
+                space_count += 1
+            else:
+                _pref = self.text[:self.index].rstrip()
+
+                if _pref == self.prefix and space_count <= 1:
+                    self.args = self.text[self.index:].lstrip()
+
+                    break
+
+            self._shift()
+
+
 class Cmd:
     """main class for parsing commands"""
 
     def __init__(self, command_string, prefix="/", max_args=0, convert_args=False):
-        self.parsed_command = None
         self.name = None
         self.args = []
         self.args_count = len(self.args)
@@ -56,53 +87,40 @@ class Cmd:
         self.max_args = max_args
 
         # parse command
-        res = re.match(rf"^{self.prefix}(?P<args>.*)", self.command_string)
+        res = Parser(self.command_string, self.prefix)
         argres = []
-        if res is not None:
-            argres = shlex.split("".join(res.group("args")))
-        argsc = len(argres)
+        if res.args is not None:
+            argres = shlex.split(res.args)
 
         if self.max_args == 0:
-            self.max_args = argsc
+            self.max_args = len(argres) - 1 if (len(argres) - 1) >= 0 else 0
 
-        if argsc > self.max_args:
-            raise ParsingError(f"arguments exceeds max arguments: ({self.max_args})")
+        if (len(argres) - 1) >= 0 and (len(argres) - 1) > self.max_args:
+            raise ParsingError(f"arguments exceeds max arguments: {self.max_args}")
 
-        for i in range(len(argres), self.max_args):  # insert empty arguments
-            argres.insert(i, "")
+        while (len(argres) - 1) >= 0 and (len(argres) - 1) < self.max_args:  # insert empty arguments
+            argres.append("")
 
         if argres:
-            self.parsed_command = {
-                "name": argres[0],
-                "args": argres[1:],
-                "args_count": argsc,
-            }
+            self.name = argres[0]
+            self.args = argres[1:]
+            self.args_count = len(argres[1:])
 
             if convert_args:
                 self._cvt_cmd()
 
-            self.name = self.parsed_command["name"]
-            self.args = self.parsed_command["args"]
-            self.args_count = self.parsed_command["args_count"]
-
-    def get_dict(self):
-        """return parsed command"""
-        return self.parsed_command
-
     def _get_args_type_char(self, max_args=0):
         """get command arguments data types in char format"""
         argtype = list()
 
         if max_args == 0:
-            for arg in self.parsed_command["args"][
-                0 : self.parsed_command["args_count"]
-            ]:
+            for arg in self.args[0: len(self.args)]:
                 if not arg:
                     continue
 
                 argtype.append(type(arg).__name__[0])  # get type char
         else:
-            for arg in self.parsed_command["args"][0:max_args]:
+            for arg in self.args[0:max_args]:
                 if not arg:
                     continue
 
@@ -114,16 +132,16 @@ class Cmd:
         """evaluate literal arguments"""
         cvt = [(_CVT_FLOAT_PTR, float), (_CVT_INT_PTR, int)]
 
-        for i in range(len(self.parsed_command["args"])):
-            if not self.parsed_command["args"][i]:
+        for i in range(len(self.args)):
+            if not self.args[i]:
                 break  # empty args
 
             for cvt_ in cvt:
-                res = cvt_[0].match(self.parsed_command["args"][i])
+                res = cvt_[0].match(self.args[i])
 
                 if res:
-                    self.parsed_command["args"][i] = cvt_[1](
-                        self.parsed_command["args"][i]
+                    self.args[i] = cvt_[1](
+                        self.args[i]
                     )
                     break  # has found the correct data type
 
@@ -152,7 +170,7 @@ class Cmd:
         """match arguments by arguments data types"""
         matched = 0
         for i, arg_type in enumerate(argtype):
-            arg_len = len(str(self.parsed_command["args"][i]))
+            arg_len = len(str(self.args[i]))
             if arg_type in ("i", "f"):
                 if format_match[i] == "s":
                     matched += 1  # allow int or float as 's' format
@@ -171,7 +189,7 @@ class Cmd:
 
         return False
 
-    def process_cmd(self, callback, error_handler_callback=None, attrs=None):
+    def process_cmd(self, callback, error_callback=None, attrs=None):
         """process command..."""
         if attrs is None:
             attrs = {}
@@ -182,8 +200,8 @@ class Cmd:
         ):
             raise TypeError("callback is not a function or method")
         if (
-            error_handler_callback
-            and inspect.isfunction(error_handler_callback) is False
+            error_callback
+            and inspect.isfunction(error_callback) is False
             and inspect.ismethod(callback) is False
         ):
             raise TypeError("error handler callback is not a function")
@@ -203,16 +221,16 @@ class Cmd:
                 if hasattr(callback.__self__, attr):
                     cdefattr.update({attr: getattr(callback.__self__, attr)})
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in attrs:
-                    if hasattr(error_handler_callback, attr):
-                        cedefattr.update({attr: getattr(error_handler_callback, attr)})
+                    if hasattr(error_callback, attr):
+                        cedefattr.update({attr: getattr(error_callback, attr)})
             else:
                 for attr in attrs:
-                    if hasattr(error_handler_callback.__self__, attr):
+                    if hasattr(error_callback.__self__, attr):
                         cedefattr.update(
-                            {attr: getattr(error_handler_callback.__self__, attr)}
+                            {attr: getattr(error_callback.__self__, attr)}
                         )
 
         if not inspect.ismethod(callback):
@@ -222,13 +240,13 @@ class Cmd:
             for attr in attrs:
                 setattr(callback.__self__, attr, attrs[attr])
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in attrs:
-                    setattr(error_handler_callback, attr, attrs[attr])
+                    setattr(error_callback, attr, attrs[attr])
             else:
                 for attr in attrs:
-                    setattr(error_handler_callback.__self__, attr, attrs[attr])
+                    setattr(error_callback.__self__, attr, attrs[attr])
 
         ret = None
         try:
@@ -242,48 +260,39 @@ class Cmd:
                     cparams = cparams[1:]
 
             if cdefaults is None:
-                if len(self.parsed_command["args"]) < len(cparams):
+                if len(self.args) < len(cparams):
                     raise MissingRequiredArgument(
                         "missing required argument: "
-                        + cparams[len(self.parsed_command["args"])],
-                        param=cparams[len(self.parsed_command["args"])],
+                        + cparams[len(self.args)],
+                        param=cparams[len(self.args)],
                     )
 
             else:
 
                 posargs_length = len(cparams) - len(cdefaults)
 
-                if len(self.parsed_command["args"]) < posargs_length:
+                if len(self.args) < posargs_length:
                     raise MissingRequiredArgument(
                         "missing required argument: "
-                        + cparams[len(self.parsed_command["args"])],
-                        param=cparams[len(self.parsed_command["args"])],
+                        + cparams[len(self.args)],
+                        param=cparams[len(self.args)],
                     )
 
-                if (len(self.parsed_command) - posargs_length) < len(cdefaults):
-                    for darg in cdefaults:
-                        self.parsed_command["args_count"] += 1
-                        self.parsed_command["args"].insert(
-                            self.parsed_command["args_count"], darg
-                        )
-
             if cargspec.varargs is None:
-                ret = callback(*self.parsed_command["args"][: len(cparams)])
+                ret = callback(*self.args[: len(cparams)])
             else:
-                ret = callback(
-                    *self.parsed_command["args"][: self.parsed_command["args_count"]]
-                )
+                ret = callback(*self.args)
 
         except Exception as exception:
-            if error_handler_callback is None:
+            if error_callback is None:
                 raise ProcessError(
                     "an error occurred during processing callback '"
-                    + f"{callback.__name__}()' for command '{self.parsed_command['name']}, "
-                    + "no error handler callback specified, exception: ",
+                    + f"{callback.__name__}()' for command '{self.name}, "
+                    + "no error handler callback specified.",
                     exception,
                 ) from exception
 
-            error_handler_callback(error=exception)
+            error_callback(error=exception)
 
         if not inspect.ismethod(callback):
             for attr in attrs:
@@ -294,15 +303,15 @@ class Cmd:
                 if hasattr(callback.__self__, attr):
                     delattr(callback.__self__, attr)
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in attrs:
-                    if hasattr(error_handler_callback, attr):
-                        delattr(error_handler_callback, attr)
+                    if hasattr(error_callback, attr):
+                        delattr(error_callback, attr)
             else:
                 for attr in attrs:
-                    if hasattr(error_handler_callback.__self__, attr):
-                        delattr(error_handler_callback.__self__, attr)
+                    if hasattr(error_callback.__self__, attr):
+                        delattr(error_callback.__self__, attr)
 
         if not inspect.ismethod(callback):
             for attr in cdefattr:
@@ -311,13 +320,13 @@ class Cmd:
             for attr in cdefattr:
                 setattr(callback.__self__, attr, cdefattr[attr])
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in cedefattr:
-                    setattr(error_handler_callback, attr, cedefattr[attr])
+                    setattr(error_callback, attr, cedefattr[attr])
             else:
                 for attr in cedefattr:
-                    setattr(error_handler_callback.__self__, attr, cedefattr[attr])
+                    setattr(error_callback.__self__, attr, cedefattr[attr])
 
         return ret
 
@@ -331,7 +340,7 @@ class Cmd:
 
         return message
 
-    async def aio_process_cmd(self, callback, error_handler_callback=None, attrs=None):
+    async def aio_process_cmd(self, callback, error_callback=None, attrs=None):
         """coroutine process cmd"""
         if attrs is None:
             attrs = {}
@@ -339,8 +348,8 @@ class Cmd:
         if inspect.iscoroutinefunction(callback) is False:
             raise TypeError("callback is not a coroutine function")
         if (
-            error_handler_callback
-            and inspect.iscoroutinefunction(error_handler_callback) is False
+            error_callback
+            and inspect.iscoroutinefunction(error_callback) is False
         ):
             raise TypeError(
                 "error handler callback is not a coroutine function function"
@@ -361,16 +370,16 @@ class Cmd:
                 if hasattr(callback.__self__, attr):
                     cdefattr.update({attr: getattr(callback.__self__, attr)})
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in attrs:
-                    if hasattr(error_handler_callback, attr):
-                        cedefattr.update({attr: getattr(error_handler_callback, attr)})
+                    if hasattr(error_callback, attr):
+                        cedefattr.update({attr: getattr(error_callback, attr)})
             else:
                 for attr in attrs:
-                    if hasattr(error_handler_callback.__self__, attr):
+                    if hasattr(error_callback.__self__, attr):
                         cedefattr.update(
-                            {attr: getattr(error_handler_callback.__self__, attr)}
+                            {attr: getattr(error_callback.__self__, attr)}
                         )
 
         if not inspect.ismethod(callback):
@@ -380,13 +389,13 @@ class Cmd:
             for attr in attrs:
                 setattr(callback.__self__, attr, attrs[attr])
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in attrs:
-                    setattr(error_handler_callback, attr, attrs[attr])
+                    setattr(error_callback, attr, attrs[attr])
             else:
                 for attr in attrs:
-                    setattr(error_handler_callback.__self__, attr, attrs[attr])
+                    setattr(error_callback.__self__, attr, attrs[attr])
 
         ret = None
         try:
@@ -400,48 +409,39 @@ class Cmd:
                     cparams = cparams[1:]
 
             if cdefaults is None:
-                if len(self.parsed_command["args"]) < len(cparams):
+                if len(self.args) < len(cparams):
                     raise MissingRequiredArgument(
                         "missing required argument: "
-                        + cparams[len(self.parsed_command["args"])],
-                        param=cparams[len(self.parsed_command["args"])],
+                        + cparams[len(self.args)],
+                        param=cparams[len(self.args)],
                     )
 
             else:
 
                 posargs_length = len(cparams) - len(cdefaults)
 
-                if len(self.parsed_command["args"]) < posargs_length:
+                if len(self.args) < posargs_length:
                     raise MissingRequiredArgument(
                         "missing required argument: "
-                        + cparams[len(self.parsed_command["args"])],
-                        param=cparams[len(self.parsed_command["args"])],
+                        + cparams[len(self.args)],
+                        param=cparams[len(self.args)],
                     )
 
-                if (len(self.parsed_command) - posargs_length) < len(cdefaults):
-                    for darg in cdefaults:
-                        self.parsed_command["args_count"] += 1
-                        self.parsed_command["args"].insert(
-                            self.parsed_command["args_count"], darg
-                        )
-
             if cargspec.varargs is None:
-                ret = await callback(*self.parsed_command["args"][: len(cparams)])
+                ret = await callback(*self.args[: len(cparams)])
             else:
-                ret = await callback(
-                    *self.parsed_command["args"][: self.parsed_command["args_count"]]
-                )
+                ret = await callback(*self.args)
 
         except Exception as exception:
-            if error_handler_callback is None:
+            if error_callback is None:
                 raise ProcessError(
                     "an error occurred during processing callback '"
-                    + f"{callback.__name__}()' for command '{self.parsed_command['name']}, "
+                    + f"{callback.__name__}()' for command '{self.name}, "
                     + "no error handler callback specified, exception: ",
                     exception,
                 ) from exception
 
-            await error_handler_callback(error=exception)
+            await error_callback(error=exception)
 
         if not inspect.ismethod(callback):
             for attr in attrs:
@@ -452,15 +452,15 @@ class Cmd:
                 if hasattr(callback.__self__, attr):
                     delattr(callback.__self__, attr)
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in attrs:
-                    if hasattr(error_handler_callback, attr):
-                        delattr(error_handler_callback, attr)
+                    if hasattr(error_callback, attr):
+                        delattr(error_callback, attr)
             else:
                 for attr in attrs:
-                    if hasattr(error_handler_callback.__self__, attr):
-                        delattr(error_handler_callback.__self__, attr)
+                    if hasattr(error_callback.__self__, attr):
+                        delattr(error_callback.__self__, attr)
 
         if not inspect.ismethod(callback):
             for attr in cdefattr:
@@ -469,12 +469,12 @@ class Cmd:
             for attr in cdefattr:
                 setattr(callback.__self__, attr, cdefattr[attr])
 
-        if error_handler_callback is not None:
-            if not inspect.ismethod(error_handler_callback):
+        if error_callback is not None:
+            if not inspect.ismethod(error_callback):
                 for attr in cedefattr:
-                    setattr(error_handler_callback, attr, cedefattr[attr])
+                    setattr(error_callback, attr, cedefattr[attr])
             else:
                 for attr in cedefattr:
-                    setattr(error_handler_callback.__self__, attr, cedefattr[attr])
+                    setattr(error_callback.__self__, attr, cedefattr[attr])
 
         return ret
